@@ -27,13 +27,11 @@ except:
     basestring = unicode = str
 
 import warnings
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import BytesIO as StringIO
+from io import BytesIO
 import xml.etree.ElementTree as et
 
 from .specs import SPECS, REVERSE_SPECS
+from .wcs_utils import get_cdelt_crota, get_cd
 
 
 def register_namespace(tag, uri):
@@ -364,18 +362,29 @@ class AVM(AVMContainer):
             return object.__getattr__(self, attribute)
 
     @classmethod
-    def from_image(cls, filename):
+    def from_image(cls, filename, xmp_packet_index=None):
         """
         Instantiate an AVM object from an existing image.
+
+        Parameters
+        ----------
+        filename : str
+            The filename of the image to read the AVM meta-data from
+        xmp_packet_index : int, optional
+            In cases where multiple XMP packets are present in the file, this
+            can be used to indicate which one to use. If not specified, this
+            defaults to the first XMP packet found.
         """
 
         # Get XMP data from file
-        xmp = extract_xmp(filename)
+        xmp = extract_xmp(filename, xmp_packet_index=xmp_packet_index)
+
+        print(xmp)
 
         # Extract XML
-        start = xmp.index("<?xpacket begin=")
-        start = xmp.index("?>", start) + 2
-        end = xmp.index("</x:xmpmeta>") + 12
+        start = xmp.index(b"<?xpacket begin=")
+        start = xmp.index(b"?>", start) + 2
+        end = xmp.index(b"</x:xmpmeta>") + 12
 
         # Extract XML
         xml = xmp[start:end]
@@ -397,7 +406,7 @@ class AVM(AVMContainer):
         self = cls()
 
         # Parse XML
-        tree = et.parse(StringIO(xml))
+        tree = et.parse(BytesIO(xml))
         root = tree.getroot()
         avm_content = parse_avm_content(root)
 
@@ -452,7 +461,7 @@ class AVM(AVMContainer):
 
         if use_full_header and self.Spatial.FITSheader is not None:
             print("Using full FITS header from Spatial.FITSheader")
-            header = fits.Header(txtfile=StringIO(self.Spatial.FITSheader))
+            header = fits.Header(txtfile=BytesIO(self.Spatial.FITSheader))
             return WCS(header)
 
         # Initializing WCS object
@@ -626,12 +635,14 @@ class AVM(AVMContainer):
         self.Spatial.ReferenceValue = wcs.wcs.crval.tolist()
         self.Spatial.ReferencePixel = wcs.wcs.crpix.tolist()
 
-        # The following is required to cover all cases of CDELT/PC/CD
-        import numpy as np  # will be installed if Astropy is present
-        cdelt = np.matrix(wcs.wcs.get_cdelt())
-        pc = np.matrix(wcs.wcs.get_pc())
-        scale = np.array(cdelt * pc)[0,:].tolist()
-        self.Spatial.Scale = scale
+        try:
+            cdelt1, cdelt2, crota2 = get_cdelt_crota(wcs)
+            self.Spatial.Scale = cdelt1, cdelt2
+            self.Spatial.Rotation = crota2
+        except ValueError:
+            warnings.warn("cannot represent WCS as a Scale and Rotation, using deprecated CDMatrix instead")
+            cd = get_cd(wcs)
+            self.Spatial.CDMatrix = cd
 
         xcoord = decode_ascii(wcs.wcs.ctype[0][:4])
         ycoord = decode_ascii(wcs.wcs.ctype[1][:4])
@@ -651,11 +662,6 @@ class AVM(AVMContainer):
         else:
             raise Exception("Unknown coordinate system: {0}/{1}".format(xcoord, ycoord))
 
-        try:
-            self.Spatial.Rotation = wcs.wcs.crota[1]
-        except:
-            pass
-
         self.Spatial.Quality = "Full"
 
         return self
@@ -697,8 +703,8 @@ class AVM(AVMContainer):
         # Create XML Tree
         tree = et.ElementTree(root)
 
-        # Need to create a StringIO object to write to
-        s = StringIO()
+        # Need to create a BytesIO object to write to
+        s = BytesIO()
         tree.write(s, encoding='utf-8')
 
         # Rewind and read the contents
